/*
 * This file is part of the Disco Deterministic Network Calculator v2.1.0 "Hydra".
 *
 * Copyright (C) 2013 - 2015 Steffen Bondorf
 *
 * disco | Distributed Computer Systems Lab
 * University of Kaiserslautern, Germany
 *
 * http://disco.cs.uni-kl.de
 *
 *
 * The Disco Deterministic Network Calculator (DiscoDNC) is free software;
 * you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 */

package unikl.disco.nc;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * 
 * @author Steffen Bondorf
 *
 */
public final class Configuration {
	private static boolean use_short_alias = true;

	private static boolean arrival_curve_checks = true;
	private static boolean service_curve_checks = true;
	private static boolean fifo_mux_checks = true;
	private static boolean deconvolution_checks = true;

	public enum MuxDiscipline { SERVER_LOCAL, GLOBAL_ARBITRARY, GLOBAL_FIFO }

	public enum GammaFlag { SERVER_LOCAL, GLOBALLY_ON, GLOBALLY_OFF }

	private static MuxDiscipline multiplexing_discipline = MuxDiscipline.SERVER_LOCAL;

	private static GammaFlag use_gamma = GammaFlag.SERVER_LOCAL;

	private static GammaFlag use_extra_gamma = GammaFlag.SERVER_LOCAL;

	public enum ArrivalBoundMethods 
	{ PBOO_PER_HOP, PBOO_CONCATENATION, PBOO_OUTPUT_AC, PMOO,
		PER_FLOW_SFA, PER_FLOW_PMOO }

	private static Set<ArrivalBoundMethods> arrival_bound_methods = new HashSet<ArrivalBoundMethods>( Collections.singleton( ArrivalBoundMethods.PBOO_CONCATENATION ) );

	private static boolean remove_duplicate_arrival_bounds = true;
	
	public static boolean useShortAliases()
	{
		return use_short_alias;
	}

	public static void setUseShortAliases( boolean short_alias )
	{
		use_short_alias = short_alias;
	}

	public static boolean performArrivalCurveChecks()
	{
		return arrival_curve_checks;
	}

	public static void setPerformArrivalCurveChecks( boolean checks )
	{
		arrival_curve_checks = checks;
	}

	public static boolean performServiceCurveChecks()
	{
		return service_curve_checks;
	}

	public static void setPerformServiceCurveChecks( boolean checks )
	{
		service_curve_checks = checks;
	}

	public static boolean performFifoMultiplexingChecks()
	{
		return fifo_mux_checks;
	}

	public static void setPerformFifoMultiplexingChecks( boolean checks )
	{
		fifo_mux_checks = checks;
	}

	public static boolean performDeconvolutionChecks()
	{
		return deconvolution_checks;
	}

	public static void setPerformDeconvolutionChecks( boolean checks )
	{
		deconvolution_checks = checks;
	}
	
	public static MuxDiscipline multiplexingDiscipline()
	{
		return multiplexing_discipline;
	}
	
	public static void setMultiplexingDiscipline( MuxDiscipline mux_discipline )
	{
		multiplexing_discipline = mux_discipline;
	}	
	
	public static GammaFlag useGamma()
	{
		return use_gamma;
	}
	
	public static void setUseGamma( GammaFlag use_gamma_flag )
	{
		use_gamma = use_gamma_flag;
	}
	
	public static GammaFlag useExtraGamma()
	{
		return use_extra_gamma;
	}
	
	public static void setUseExtraGamma( GammaFlag use_extra_gamma_flag )
	{
		use_extra_gamma = use_extra_gamma_flag;
	}
	
	public static Set<ArrivalBoundMethods> arrivalBoundMethods()
	{
		return arrival_bound_methods;
	}
	
	public static void defaultArrivalBoundMethods()
	{
		arrival_bound_methods.clear();
		arrival_bound_methods.add( ArrivalBoundMethods.PBOO_CONCATENATION );
	}
	
	public static void setArrivalBoundMethod( ArrivalBoundMethods arrival_bound_method )
	{
		arrival_bound_methods.clear();
		arrival_bound_methods.add( arrival_bound_method );
	}

	public static void setArrivalBoundMethods( Set<ArrivalBoundMethods> arrival_bound_methods_set )
	{
		arrival_bound_methods.clear();
		arrival_bound_methods.addAll( arrival_bound_methods_set );
	}

	public static void addArrivalBoundMethod( ArrivalBoundMethods arrival_bound_method )
	{
		arrival_bound_methods.add( arrival_bound_method );
	}
	
	public static void addArrivalBoundMethods( Set<ArrivalBoundMethods> arrival_bound_methods_set )
	{
		arrival_bound_methods.addAll( arrival_bound_methods_set );
	}
	
	public static boolean removeDuplicateArrivalBounds()
	{
		return remove_duplicate_arrival_bounds;
	}
	
	public static void setRemoveDuplicateArrivalBounds( boolean remove_duplicate_arrival_bounds_flag )
	{
		remove_duplicate_arrival_bounds = remove_duplicate_arrival_bounds_flag;
	}
}
